turtles-own [energy speed movement detection] ;; Properties/attributes the turtles have
globals [days turns amount-grass half-patches turtle-count] ;; sliders: food-energy speed-cost birth-requirement base-movement-cost starting-energy default turns-in-day




to setup
  clear-all
  setup-patches
  setup-turtles
  reset-ticks
end

to go
  check-death-energy
  count-turtles
  move-turtles
  eat-grass
  set-label
  tick
  reset-at-50
  reset-at-max
  end

to setup-defaults
  set food-energy 30
  set speed-cost 1.04
  set birth-requirement 200
  set base-movement-cost 5
  set starting-energy 25
  set turns-in-day 10
  set starting-speed 2
  set detection-cost 0.9
  set initial-population 30
  set two-populations false
  set migration false
end

to setup-patches
  ask patches [set pcolor brown] ;;makes all patches brown
  set amount-grass (precision (count patches / 2) -1) ;set default amount of grass to half of the total patches rounded down
  set half-patches (precision (count patches / 2) -1)
  ask n-of amount-grass patches [ set pcolor green] ;; number of random patches set to green (grass)
  if two-populations [
    ask patches  [if (pxcor = 0) and (pycor >= 0)  [set pcolor black] ]
    ask patches  [if (pycor = 0) and (pxcor >= 0)  [set pcolor black] ]]
end

to setup-turtles
  create-turtles initial-population    ;; Create a number of turtles at start
  ask turtles [
    setxy random-pxcor random-pycor ;; set turtles in the center of random patches
    set heading one-of [0 45 90 135 180 225 270 315] ;; heading in 1 of the 8 directions
    set energy starting-energy
    set speed starting-speed ;; sets speed as starting-speed
    set detection 1
    set movement speed ;; Sets movement to the speed of the turtle
    if Labels = "speed" [set label speed]
    if Labels = "movement" [set label movement]
    set color one-of [15 45 85 115 125]
      ]
  set turns 0
  set days 0
end

to check-death-energy
  ask turtles[if energy >= birth-requirement [set movement 0]]
  ask turtles[if energy < 0 [set energy 0]]
  ask turtles [if energy = 0 [die]]
  ask turtles [if pcolor = black [die]]
end

to count-turtles
  set turtle-count (count turtles)
end

to set-label
  ask turtles [
    if Labels = "speed" [set label speed]
    if Labels = "movement" [set label movement]]
end

to move-turtles
  ask turtles [
      if (energy > 0) and (energy < birth-requirement) [
      ifelse (any? patches with [pcolor = green] in-radius detection) and (detection > 0)
      [face min-one-of patches with [pcolor = green] [distance myself]]
      [set heading (heading + one-of [-45 -90 45 90 0])]
      if movement > 0 [
        if (is-patch? patch-ahead 1) and (two-populations) [
        ifelse  [pcolor] of patch-ahead 1 = black
        [set heading (heading + one-of [-90 90])
            if is-patch? patch-ahead 1 [if [pcolor] of patch-ahead 1 != black [forward 1]]]
          [if is-patch? patch-ahead 1 [if [pcolor] of patch-ahead 1 != black [forward 1]]]
        ]
        if not two-populations [forward 1]
        setxy pxcor pycor ;;keep turtles in center of a patch
        set movement movement - 1
        set energy energy - ((base-movement-cost +  (speed-cost ^ speed)) + (detection-cost * detection ^ 2))
  ]]]
      if all? turtles [movement = 0]
      [ask turtles [set movement speed]
  set turns turns + 1]


end





to reset-at-max
  if all? turtles [energy >= birth-requirement][
    ask patches [set pcolor brown]
    set amount-grass (precision (count patches with [pcolor = brown] / 2) -1)
    ask n-of amount-grass patches [ set pcolor green]
    if  two-populations [
    ask patches  [if (pxcor = 0) and (pycor >= 0)  [set pcolor black] ]
      ask patches  [if (pycor = 0) and (pxcor >= 0)  [set pcolor black] ]]
    if (migration) and (two-populations) [
      ask patches [if (pycor = 0 ) and ((pxcor >= 10) and (pxcor <= 13)) [set pcolor one-of [green brown]]]]
    ask turtles [if energy > birth-requirement [
      set energy starting-energy
      hatch one-of [0 0 0 1] [
        set energy starting-energy
        ifelse speed >= 2
        [set speed speed + one-of [-1 1 0  ] ]
        [set speed speed + one-of [1 0]]
        ifelse detection >= 1
        [set detection detection + one-of [-1 0 0 0 1]]
          [set detection detection + one-of [0 0 0 1]]
      ]
      ]
    ]
    ask turtles [
    set energy starting-energy
    set movement speed
    ]
    set days days + 1
    reset-ticks
    set turns 0
  ]
end


to reset-at-50
   if turns > turns-in-day [
    ask patches [set pcolor brown]
    set amount-grass (precision (count patches with [pcolor = brown] / 2) -1)
    ask n-of amount-grass patches [ set pcolor green]
    if  two-populations [
    ask patches  [if (pxcor = 0) and (pycor >= 0)  [set pcolor black] ]
      ask patches  [if (pycor = 0) and (pxcor >= 0)  [set pcolor black] ]]
    if (migration) and (two-populations) [
      ask patches [if (pycor = 0 ) and ((pxcor >= 10) and (pxcor <= 13)) [set pcolor one-of [green brown]]]]
    ask turtles [if energy >= birth-requirement [
      set energy starting-energy
      hatch one-of [0 0 0 1] [
        set energy starting-energy
        ifelse speed >= 2
        [set speed speed + one-of [-1 1 0 ] ]
        [set speed speed + one-of [1 0]]
        ifelse detection >= 1
        [set detection detection + one-of [-1 0 0 1]]
          [set detection detection + one-of [0 0 1]]
      ]
      ]
    ]
    ask turtles [set energy starting-energy]
    set days days + 1
    reset-ticks
    set turns 0
  ]
end



to eat-grass
  ask turtles [
    if pcolor = green [
      set pcolor brown
      set energy energy + food-energy
    ]
  ]
end

;; unused

to reproduce
  ask turtles [
    if energy > 60 [
      set energy energy - 50
      hatch 1 [ set energy 10 ]
    ]
  ]
end

to check-death
  ask turtles [
    if energy <= 0 [ die ]
  ]
end

to regrow-grass
  ask patches [
    if random 100 < 3 [ set pcolor green ]
  ]
end

to set-color
  ask turtles[
    set color scale-color red speed 0 30]
end
@#$#@#$#@
GRAPHICS-WINDOW
232
12
872
653
-1
-1
10.361
1
10
1
1
1
0
0
0
1
-30
30
-30
30
1
1
1
ticks
30.0

BUTTON
31
57
202
120
Start/reset
setup\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
31
123
113
156
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

BUTTON
119
124
202
157
Go 1 tick
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
889
10
1387
213
Population size
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot count turtles"

PLOT
1177
348
1388
468
Energy
NIL
NIL
0.0
210.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [energy] of turtles"

PLOT
889
219
1171
469
Speed
NIL
NIL
0.0
50.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [speed] of turtles"

MONITOR
34
556
91
601
Days
days
17
1
11

MONITOR
95
556
152
601
Turns
turns
17
1
11

SLIDER
31
230
203
263
speed-cost
speed-cost
1
1.5
1.04
0.01
1
NIL
HORIZONTAL

SLIDER
31
266
203
299
food-energy
food-energy
0
50
30.0
1
1
NIL
HORIZONTAL

SLIDER
31
194
203
227
birth-requirement
birth-requirement
0
300
200.0
1
1
NIL
HORIZONTAL

SLIDER
31
159
203
192
base-movement-cost
base-movement-cost
0
30
5.0
1
1
NIL
HORIZONTAL

SLIDER
32
335
204
368
starting-energy
starting-energy
1
50
25.0
1
1
NIL
HORIZONTAL

SLIDER
32
371
204
404
turns-in-day
turns-in-day
0
50
10.0
1
1
NIL
HORIZONTAL

MONITOR
130
607
225
652
Population size
turtle-count
17
1
11

PLOT
1177
219
1388
349
Detection
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [detection] of turtles"

SLIDER
32
441
204
474
detection-cost
detection-cost
0
3
0.9
.1
1
NIL
HORIZONTAL

SLIDER
32
406
204
439
starting-speed
starting-speed
1
30
2.0
1
1
NIL
HORIZONTAL

CHOOSER
33
604
125
649
Labels
Labels
"speed" "movement"
0

PLOT
889
475
1390
625
Colours
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Red" 1.0 0 -2674135 true "" "plotxy days count turtles with [color = 15]\nif days > 40                               \n[set-plot-x-range (days - 40) days]"
"Yellow" 1.0 0 -1184463 true "" "plotxy days count turtles with [color = 45]"
"Cyan" 1.0 0 -11221820 true "" "plotxy days count turtles with [color = 85]"
"Violet" 1.0 0 -8630108 true "" "plotxy days count turtles with [color = 115]"
"Magenta" 1.0 0 -5825686 true "" "plotxy days count turtles with [color = 125]"

SLIDER
31
300
203
333
initial-population
initial-population
0
100
30.0
1
1
NIL
HORIZONTAL

SWITCH
34
517
206
550
migration
migration
1
1
-1000

SWITCH
34
480
205
513
two-populations
two-populations
1
1
-1000

BUTTON
31
13
202
53
Setup defaults
setup-defaults
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
